'''Rainbows full of color :-)))'''

import random


class Color(object):
    def __init__(self, red, green, blue):
        self._red   = max(0, min(255, red))
        self._green = max(0, min(255, green))
        self._blue  = max(0, min(255, blue))
        
    def red(self):
        return self._red
    
    def green(self):
        return self._green
    
    def blue(self):
        return self._blue
    
    def tuple(self):
        return (self.red(), self.green(), self.blue())
    
    def shifted_randomly(self, d):
        (red, green, blue) = map(lambda x: random.random() * x * d + x * (1 - d),
                                 self.tuple())
        
        return Color(red, green, blue)
    
    def dimmed(self):
        (red, green, blue) = map(lambda x: x / 2, self.tuple())
        
        return Color(red, green, blue)
    
    def brightened(self):
        (red, green, blue) = map(lambda x: (255 + x * 3) / 4, self.tuple())
        
        return Color(red, green, blue)
    
    def mix(self, color, factor = 0.5):
        (red, green, blue) = map(lambda x, y: int(x * (1 - factor) + y * factor),
                                 self.tuple(), color.tuple())
        
        return Color(red, green, blue)
        
    def __eq__(self, other):
        return (other is not None and
                self.__class__ == other.__class__ and
                self.red() == other.red() and
                self.green() == other.green() and
                self.blue() == other.blue())
    
    def __hash__(self):
        return self.red() + self.green() + self.blue()
    
    def __repr__(self):
        return 'Color' + str(self.tuple())
    
    def __str__(self):
        return '#%02x%02x%02x' % self.tuple()
    