'''Basic geometry stuff.'''

from math import *


class Vector(object):
    def __init__(self, x, y):
        self._x = x
        self._y = y
        
    def x(self):
        return self._x
    
    def y(self):
        return self._y
    
    def tuple(self):
        return (self.x(), self.y())
    
    def length(self):
        return sqrt(self.x() * self.x() + self.y() * self.y())
    
    def int_tuple(self):
        return (int(self.x()), int(self.y()))
    
    def translated(self, vector):
        return Vector(self.x() + vector.x(), self.y() + vector.y())
    
    def scaled(self, vector):
        return Vector(self.x() * vector.x(), self.y() * vector.y())
    
    def vector(self):
        return self
    
    __add__ = translated
    __mul__ = scaled
    
    def __sub__(self, other):
        return self + -other
    
    def __div__(self, number):
        return Vector(self.x() / number, self.y() / number)
    
    def __neg__(self):
        return self * Vector(-1, -1)
    
    def __eq__(self, other):
        return (other is not None and
                self.__class__ == other.__class__ and
                self.x() == other.x() and
                self.y() == other.y())
    
    def __ne__(self, other):
        return not self == other
    
    def __hash__(self):
        return self.x() + self.y()
    
    def __repr__(self):
        return 'Vector' + str(self)
    
    def __str__(self):
        return str(self.tuple())


class Area(object):
    def __init__(self, origin, extent):
        self._origin = origin
        self._extent = extent
        
    def origin(self):
        return self._origin
    
    def extent(self):
        return self._extent
    
    def far_point(self):
        return self.origin() + self.extent() - Vector(1, 1)
    
    def includes(self, vector):    
        return (self.origin().x() <= vector.x() <= self.far_point().x() and
                self.origin().y() <= vector.y() <= self.far_point().y())
        
    def shrunken(self, dx, dy = None):
        if dy is None:
            dy = dx
        
        return self.grown(-dx, -dy)
    
    def grown(self, dx, dy = None):
        if dy is None:
            dy = dx
        
        return Area(self.origin(), self.extent() + Vector(dx, dy))
    
    def moved(self, dx, dy = None):
        if dy is None:
            dy = dx
        
        return Area(self.origin() + Vector(dx, dy), self.extent())
    
    def cropped(self, dx1, dy1 = None, dx2 = None, dy2 = None):
        if dy1 is None:
            dy1 = dx1
        
        if dx2 is None:
            dx2 = dx1
        
        if dy2 is None:
            dy2 = dy1
        
        return self.moved(dx1, dy1).shrunken(dx1 + dx2, dy1 + dy2)
    
    def with_width(self, width):
        return Area(self.origin(), Vector(width, self.extent().y()))
    
    def with_height(self, height):
        return Area(self.origin(), Vector(self.extent().x(), height))

    def area(self):
        return self.extent().x() * self.extent().y()
        
    def __eq__(self, other):
        return (other is not None and
                self.__class__ == other.__class__ and
                self.origin() == other.origin() and
                self.extent() == other.extent())
    
    def __lt__(self, other):
        return self.area() < other.area()
    
    def __hash__(self):
        return self.origin().__hash__() + self.extent().__hash__()

    def __repr__(self):
        return 'Area(%s, %s)' % (repr(self.origin()), repr(self.extent()))
    
    def __str__(self):
        return '%s-%s' % (str(self.origin()), str(self.far_point()))
    