'''Ugh, work!'''

from concepts.color import *
            
from move import *

import random


class JobBoard(object):
    def __init__(self, city):
        self._city             = city
        self._available_jobs   = set()
        self._unavailable_jobs = set()
        
    def jobs(self):
        return set(self._available_jobs + self._unavailable_jobs)
        
    def available_jobs(self):
        return set(self._available_jobs)
        
    def unavailable_jobs(self):
        return set(self._unavailable_jobs)
    
    def set_available(self, job):
        self._available_jobs.add(job)
        self._unavailable_jobs.discard(job)
        
    def set_unavailable(self, job):
        self._available_jobs.discard(job)
        self._unavailable_jobs.add(job)
    
    def add_job(self, job):
        if job.is_available():
            self.set_available(job)
        else:
            self.set_unavailable(job)
    
    def cancel_job(self, job):
        job.cancel()
        self.remove_job(job)
        
    def remove_job(self, job):
        self._available_jobs.discard(job)
        self._unavailable_jobs.discard(job)
        
    def seek_employment(self, person):
        jobs = self.available_jobs()
        
        if not jobs:
            return
        
        random.choice(list(jobs)).employ(person)
                    
            
class Job(object):
    def __init__(self, building):
        self._building = building
        self._person   = None
        self._active   = True
        
    def title(self):
        return 'Worker'
    
    def color(self):
        return Color(255, 0, 0)
    
    def building(self):
        return self._building
    
    def person(self):
        return self._person
    
    def plan(self):
        return GoAndDoJob(self.person(), self)
    
    def has_plan(self):
        return True
    
    def office(self):
        return random.choice(list(self.building().tiles()))
    
    def target(self):
        return random.choice(list(self.building().tiles()))
    
    def work_done(self):
        pass
    
    def work_speed(self):
        return 0.02
    
    def employ(self, person):
        if self.person() == person:
            return
        
        self.fire_worker()
        self._person = person
        
        if person is not None:
            self.building().city().jobs().set_unavailable(self)
            self.person().employed(self)
            
    def fire_worker(self):
        if self.person() is None:
            return
        
        self.person().fired(self)
        self._person = None
        self.building().city().jobs().set_available(self)
    
    def cancel(self):
        self.fire_worker()
        self._active = False
        
    def is_active(self):
        return self._active
    
    def is_available(self):
        return (self._person is None) and self.is_active()

    def __str__(self):
        return self.__class__.__name__
    
    
class Production(Action):
    def __init__(self, person, job):
        super(Production, self).__init__(person)
        
        self._job    = job
        self._progress = 0.0
    
    def speed(self):
        return self.job().work_speed()
    
    def job(self):
        return self._job
    
    def progress(self):
        return self._progress
    
    def unit_produced(self):
        self.job().work_done()
    
    def act(self):
        self._progress += self.speed()
        
        if self.progress() >= 1.0:
            self.unit_produced()
            self.done()


class GoAndDoJob(Plan):
    def __init__(self, person, job, resource = None):
        super(GoAndDoJob, self).__init__(person)
        
        self._job    = job
        self._routes = [lambda: Route(person, job.office()),
                        lambda: Route(person, job.target()),
                        lambda: Route(person, job.office())]
        self._stage  = 0
        self._action = Production(person, job)
    
    def color(self):
        return Color(80 * self.stage(), 80 * self.stage(), 255)
        
    def job(self):
        return self._job
    
    def stage(self):
        return self._stage
    
    def route(self):
        if callable(self._routes[self.stage()]):
            self._routes[self.stage()] = self._routes[self.stage()]()
            
        return self._routes[self.stage()]
    
    def action(self):
        return self._action
    
    def next_action(self):
        action = self.route().next_action()
        
        if action is None:
            if self.stage() == 1:
                action = self.action()
                self._action = None
            elif self.stage() == 2:
                return None
            
        if action is None:
            self._stage += 1            
            return self.next_action()
        
        return action

    def path(self):
        if callable(self._routes[self.stage()]):
            return []
        
        return self.route().path()


class MoveResource(Plan):
    def __init__(self, person, resource, job, keep = False):
        super(MoveResource, self).__init__(person)
        
        if resource is not None:
            self._routes = [lambda: Route(person, job.office()),
                            lambda: Route(person, resource.place()),
                            lambda: Route(person, job.target()),
                            lambda: Route(person, job.office())]
        else:
            self._routes = [lambda: Route(person, job.office())]
            
        self._resource = resource
        self._job      = job        
        self._stage    = 0
        self._keep     = keep
    
    def keep(self):
        return self._keep
    
    def color(self):
        return Color(80 * self.stage(), 255, 80 * self.stage())
        
    def job(self):
        return self._job
    
    def stage(self):
        return self._stage
    
    def resource(self):
        return self._resource
    
    def route(self):
        if callable(self._routes[self.stage()]):
            self._routes[self.stage()] = self._routes[self.stage()]()
            
        return self._routes[self.stage()]
    
    def next_action(self):
        action = self.route().next_action()
        
        if action is None:
            if self.stage() == 1:
                self.person().take(self.resource())
                
            if self.stage() == 2:
                self.person().drop(self.resource(), self.keep())

            if self.stage() < len(self._routes) - 1:
                self._stage += 1            
                return self.next_action()

        return action

    def path(self):
        if callable(self._routes[self.stage()]):
            return []
        
        return self.route().path()
