'''Yummy yummy goods produced riverside.'''

from concepts.color import *


class Goods(object):
    def __init__(self, number, place = None):
        self._number = number
        self._place  = place
        self._owner  = None
        
    @classmethod
    def name(self):
        return ('Good', 'Goods')
    
    @classmethod
    def color(self):
        return Color(255, 0, 0)
    
    @classmethod
    def singular(self):
        return self.name()[0]
    
    @classmethod
    def plural(self):
        return self.name()[1]
    
    @classmethod
    def is_edible(self):
        return False
    
    @classmethod
    def consumption_speed(self):
        return 0.2
    
    def consumption_effect(self, person, number):
        pass
    
    def consumed_by(self, person, number):
        number = min(self.number(), number)
        
        self.decrease(number)
        self.consumption_effect(person, number)
        
    def number(self):
        return self._number
    
    def increase(self, number):
        self._number += number
        
        if self.number() <= 0:
            self.destroy()

    def decrease(self, number):
        self.increase(-number)

    def destroy(self):
        self._number = 0
        
        if self.place() is not None:
            self.place().city().consume(self)
        
    def place(self):
        return self._place
    
    def move_to(self, place):
        if place is None:
            self._place = None
        elif self.place() is None:
            self._place = place
        else:
            self.place().move_to(place.vector())
        
    def is_at(self, place):
        return (self.place() == place)
    
    def take(self, number, owner = None):
        if number >= self.number():
            number = self.number()
        
        if number <= 0:
            return None
        
        
        good = self.__class__(number)
        good.set_owner(owner)
        
        self.place().city().create(self.place(), good)
        self.decrease(number)
        
        return good
    
    def owner(self):
        return self._owner
    
    def set_owner(self, owner):
        self._owner = owner
    
    def has_owner(self):
        return (self.owner() is not None)
    
    def can_be_used_by(self, person):
        return (self.owner() is None or self.owner() == person)
    
    def __str__(self):
        if self.number() == 1:
            numerus = self.singular
        else:
            numerus = self.plural

        return '%d %s' % (self.number(), numerus().lower())


class Food(Goods):
    @classmethod
    def name(self):
        return ('Food', 'Food')
    
    @classmethod
    def is_edible(self):
        return True
    
    @classmethod
    def nutrional_value(self):
        return 0.01

    def consumption_effect(self, person, number):
        person.nutrition().add(number * self.nutrional_value())

    
    
class Wood(Goods):
    @classmethod
    def name(self):
        return ('Wood', 'Wood')
    
    @classmethod
    def color(self):
        return Color(128, 96, 0)
