'''Scenario base classes.'''


from city import *
from map import *
from time import *

class Scenario(object):
    def __init__(self):
        self._city    = None
        self._map     = None
        self._time    = Time(0.02)
        self._allowed = set()
    
    def message(self, string, color = Color(255, 255, 0)):
        self.city().log(string, color)
    
    def hint(self, string, color = Color(128, 128, 128)):
        self.city().log('  ' + string, color)
        
    def is_allowed(self, type):
        return not self._allowed or type in self._allowed

    def allow(self, type):
        self._allowed.add(type)
        
    def time(self):
        return self._time
    
    def map_size(self):
        return Vector(25, 25)
        
    def new_map(self):
        return CityMap(self.map_size())
    
    def map(self):
        if self._map is None:
            self._map = self.new_map()
    
        return self._map
    
    def city_name(self):
        return None
    
    def new_city(self):
        return City(self.city_name(), self.map(), self.time())
    
    def city(self):
        if self._city is None:
            self._city = self.new_city()
    
        return self._city
    
    def setup(self):
        pass
    
    def schedule(self):
        return {}
    
    def intro(self):
        return []
    
    def script(self):
        return []
    
    def start(self, logger = None):
        self.setup()
        self.city().set_logger(logger)
        
        for line, color in self.intro():
            self.city().log(line, color)
            
        for time in self.schedule():
            for event in self.schedule()[time]:
                self.time().do_at(time, event)
            
        for condition, event in self.script():
            self.time().do_when(condition, event)
                
        self.time().start()
    
    def stop(self):
        self.time().stop()
        