'''Main window classes.'''

import pygame

from pygame import Surface

from concepts.color import *
from concepts.geometry import *

from components import *


class Window(object):
    def __init__(self, title, extent):
        self._title      = title
        self._extent     = extent
        self._surface    = pygame.display.set_mode(extent.tuple())
        self._components = []
        
        pygame.display.set_caption(title)
        
    def title(self):
        return self._title
    
    def extent(self):
        return self._extent
    
    def surface(self):
        return self._surface
    
    def components(self):
        return list(self._components)
    
    def area(self, origin, extent):
        return SurfaceArea(origin, extent, self.surface())
    
    def full_area(self):
        return self.area(Vector(0, 0), self.extent())
    
    def add(self, component):
        self._components.append(component)
        
    def remove(self, component):
        self._components.remove(component)
        
    def flip(self):
        pygame.display.flip()
    
    def erase(self):
        self.surface().fill((0, 0, 0))

    def clear(self):
        self.erase()
        self.flip()
    
    def render(self):
        self.erase()
        
        for component in self.components():
            component.render()
        
        self.flip()
    
    def all_components(self):
        todo = self.components()
        
        for component in todo:
            if component.is_container():
                todo += component.components()
                continue
            
            yield component
        
    def do_click(self, pos, button):
        for component in self.all_components():
            if not component.area().includes(Vector(*pos)):
                continue
            
            if button == 1:
                component.do_left_click(Vector(*pos) - component.origin())
            elif button == 2:
                component.do_middle_click(Vector(*pos) - component.origin())
            elif button == 3:
                component.do_right_click(Vector(*pos) - component.origin())
    
    def do_mouse(self, pos, button):
        for component in self.all_components():
            if not component.area().includes(Vector(*pos)):
                component.do_mouse(None)
                continue
            
            component.do_mouse(Vector(*pos) - component.origin())
        
    def do_events(self):
        running = True
        
        while running:
            self.render()
            
            for event in pygame.event.get():
                if event.type == pygame.QUIT:
                    running = False
                
                if event.type == pygame.MOUSEBUTTONDOWN:
                    self.do_click(event.pos, event.button)
                
                if event.type == pygame.MOUSEMOTION:
                    self.do_mouse(event.pos, event.buttons)
                    

class PrettyWindow(Window):
    def __init__(self, title, extent, frame_color, background_color):
        super(PrettyWindow, self).__init__(title, extent)
        
        self.add(Panel(self.full_area(), frame_color, background_color, False))
        